/*  
 * Copyright (C) 2010  Lorenzo Pallara, l.pallara@avalpa.com
 * All rights reserved.
 */


#include <netinet/in.h>

#include <fcntl.h>
#include <unistd.h> 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <inttypes.h>
#include <unistd.h>

#define PACK_HEADER_SIZE 4 

int main(int argc, char *argv[])
{
	int byte_read;
	int i;
	FILE* file_input_pes;
	FILE* file_video_pes;
	FILE* file_audio_pes;
	FILE* file_output_pes = NULL;
	unsigned char pes_header[PACK_HEADER_SIZE];
	unsigned char pes_extra_header[PACK_HEADER_SIZE];
	unsigned char header_len;
	unsigned char byte;
	unsigned int pes_packet_length = 0;
	
	/* Open pes file */
	if (argc > 3) {
		file_input_pes = fopen(argv[1], "rb");
		file_audio_pes = fopen(argv[2], "wb");
		file_video_pes = fopen(argv[3], "wb");
	} else {
		fprintf(stderr, "Usage: 'pesdemux filename.pes audio.pes video.pes'\n");
		return 2;
	}
	if (file_input_pes == 0) {
		fprintf(stderr, "Can't find file %s\n", argv[1]);
		return 2;
	}
	if (file_video_pes == 0) {
		fprintf(stderr, "Can't find file %s\n", argv[3]);
		return 2;
	}
	if (file_audio_pes == 0) {
		fprintf(stderr, "Can't find file %s\n", argv[2]);
		return 2;
	}
	
	file_output_pes = NULL;
	byte_read = fread(pes_header, 1, 4, file_input_pes);
	while(byte_read) {
		if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && ((pes_header[3]) == 0x00)) {
			file_output_pes = file_video_pes;  // picture
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] >= 0x01) && (pes_header[3] <= 0xAF)) {
			file_output_pes = file_video_pes; // slice
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB0)) {
			file_output_pes = file_video_pes; // reserved
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB1)) {
			file_output_pes = file_video_pes; // reserved
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB2)) {
			file_output_pes = file_video_pes; // user data
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB3)) {
			file_output_pes = file_video_pes; // sequence header
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB4)) {
			file_output_pes = file_video_pes; // sequence error
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB5)) {
			file_output_pes = file_video_pes; // extension
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB6)) {
			file_output_pes = file_video_pes; // reserved
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB7)) {
			file_output_pes = file_video_pes; // sequence end
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB8)) {
			file_output_pes = file_video_pes; // group of pictures
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xB9)) {
			file_output_pes = NULL; // program end
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xBA)) {
			file_output_pes = NULL; // pack header
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xBB)) {
			file_output_pes = NULL; // system header
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xBC)) {
			file_output_pes = NULL; // program stream map
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xBD)) {
			file_output_pes = NULL; // private stream 1
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xBE)) {
			file_output_pes = NULL; // padding stream
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xBF)) {
			file_output_pes = NULL; // private stream 2
		} else	if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] >= 0xC0) && (pes_header[3] <= 0xDF)) {
			byte_read = fread(pes_extra_header, 1, 4, file_input_pes);
			pes_packet_length = (pes_extra_header[0] << 8) + pes_extra_header[1];
			// fprintf(stderr, "audio pes packet length %d\n", pes_packet_length);
			fread(&header_len, 1, 1, file_input_pes);
			// fprintf(stderr, "audio pes packet header length %d\n", header_len);
			for (i = 0; i < header_len; i++) {
				fread(&byte, 1, 1, file_input_pes);
			}
			pes_packet_length -= header_len;
			pes_packet_length -= 3;
			for (i = 0; i < pes_packet_length; i++) {
				fread(&byte, 1, 1, file_input_pes);
				fwrite(&byte, 1, 1, file_audio_pes);
			}
			file_output_pes = NULL;
			byte_read = fread(pes_header + 1, 1, 3, file_input_pes);
		} else	if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] >= 0xE0) && (pes_header[3] <= 0xEF)) {
			byte_read = fread(pes_extra_header, 1, 4, file_input_pes);
			pes_packet_length = (pes_extra_header[0] << 8) + pes_extra_header[1];
			// fprintf(stderr, "video pes packet length %d\n", pes_packet_length);
			fread(&header_len, 1, 1, file_input_pes);
			// fprintf(stderr, "video pes packet header length %d\n", header_len);
			for (i = 0; i < header_len; i++) {
				fread(&byte, 1, 1, file_input_pes);
			}
			pes_packet_length -= header_len;
			pes_packet_length -= 3;
			for (i = 0; i < pes_packet_length; i++) {
				fread(&byte, 1, 1, file_input_pes);
				fwrite(&byte, 1, 1, file_video_pes);
			}
			file_output_pes = NULL;
			byte_read = fread(pes_header + 1, 1, 3, file_input_pes);
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF0)) {
			file_output_pes = NULL; // ecm stream
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF1)) {
			file_output_pes = NULL; // emm stream
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF2)) {
			file_output_pes = NULL; // dsmcc
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF3)) {
			file_output_pes = NULL; // 13522
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF4)) {
			file_output_pes = NULL; // h 222.1 A
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF5)) {
			file_output_pes = NULL; // h 222.1 B
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF6)) {
			file_output_pes = NULL; // h 222.1 C
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF7)) {
			file_output_pes = NULL; // h 222.1 D
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF8)) {
			file_output_pes = NULL; // h 222.1 E
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xF9)) {
			file_output_pes = NULL; // ancillary
		} else	if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] >= 0xFA) && (pes_header[3] <= 0xFE)) {
			file_output_pes = NULL; // reserved
		} else if ((pes_header[0] == 0x00) && (pes_header[1] == 0x00) && (pes_header[2] == 0x01) && (pes_header[3] == 0xFF)) {
			file_output_pes = NULL; // directory
		}
		
		/*
		if (file_output_pes != NULL) {
			fwrite(pes_header, 1, 1, file_output_pes);
		}
		*/
		
		pes_header[0] = pes_header[1];
		pes_header[1] = pes_header[2];
		pes_header[2] = pes_header[3];
		byte_read = fread(pes_header + 3, 1, 1, file_input_pes);
	}
	
	return 0;
}


